@@ -5,6 +5,8 @@ import traceback
 
 import click
 from junitparser import JUnitXml, TestSuite
+import xml.etree.ElementTree as ET
+from typing import Callable
 
 from .case_event import CaseEvent
 from ...testpath import TestPathComponent
@@ -39,8 +41,14 @@ from .session import session
     type=str,
     metavar='BUILD_NAME'
 )
+@click.option(
+    '--debug',
+    help='print request payload',
+    default=False,
+    is_flag=True,
+)
 @click.pass_context
-def tests(context, base_path: str, session_id: str, build_name: str):
+def tests(context, base_path: str, session_id: str, build_name: str, debug: bool):
     if session_id and build_name:
         raise click.UsageError(
             'Only one of -build or -session should be specified')
@@ -58,6 +66,10 @@ def tests(context, base_path: str, session_id: str, build_name: str):
     # TODO: placed here to minimize invasion in this PR to reduce the likelihood of
     # PR merge hell. This should be moved to a top-level class
     class RecordTests:
+        # function that returns junitparser.TestCase
+        # some libraries output invalid  incorrectly format then have to fix them.
+        JUnitXmlParseFunc = Callable[[str],  ET.Element]
+
         @property
         def path_builder(self) -> CaseEvent.TestPathBuilder:
             """
@@ -70,9 +82,18 @@ def tests(context, base_path: str, session_id: str, build_name: str):
         def path_builder(self, v: CaseEvent.TestPathBuilder):
             self._path_builder = v
 
+        @property
+        def junitxml_parse_func(self):
+            return self._junitxml_parse_func
+
+        @junitxml_parse_func.setter
+        def junitxml_parse_func(self, f: JUnitXmlParseFunc):
+            self._junitxml_parse_func = f
+
         def __init__(self):
             self.reports = []
             self.path_builder = CaseEvent.default_path_builder(base_path)
+            self.junitxml_parse_func = None
 
         def make_file_path_component(self, filepath) -> TestPathComponent:
             """Create a single TestPathComponent from the given file path"""
@@ -104,8 +125,7 @@ def tests(context, base_path: str, session_id: str, build_name: str):
                 for p in self.reports:
                     # To understand JUnit XML format, https://llg.cubic.org/docs/junit/ is helpful
                     # TODO: robustness: what's the best way to deal with broken XML file, if any?
-                    xml = JUnitXml.fromfile(p)
-
+                    xml = JUnitXml.fromfile(p, self.junitxml_parse_func)
                     if isinstance(xml, JUnitXml):
                         testsuites = [suite for suite in xml]
                     elif isinstance(xml, TestSuite):
@@ -123,14 +143,12 @@ def tests(context, base_path: str, session_id: str, build_name: str):
                             yield json.dumps(CaseEvent.from_case_and_suite(self.path_builder, case, suite, p))
                 yield ']}'
 
-            # TODO: this probably should be a flag
-            if True:
-                # generator adapter that prints the content
-                def printer(f):
-                    for d in f:
-                        print(d)
-                        yield d
-                payload = printer(payload())
+            def printer(f):
+                for d in f:
+                    print(d)
+                    yield d
+
+            payload = printer(payload()) if debug else payload()
 
             # str -> bytes then gzip compress
             payload = (s.encode() for s in payload)
